@@ -1,14 +1,33 @@
-/*	$OpenBSD: history.c,v 1.34 2006/03/17 16:30:13 millert Exp $	*/
+/*	$OpenBSD: history.c,v 1.35 2006/05/29 18:22:24 otto Exp $	*/
 /*	$OpenBSD: trap.c,v 1.22 2005/03/30 17:16:37 deraadt Exp $	*/
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.15 2006/05/10 18:54:10 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.25 2006/08/24 18:57:30 tg Exp $");
 
-static int	histfd;
-static int	hsize;
+#if !defined(__sun__)
+#define DO_HISTORY
+#endif
 
-#ifndef __sun__
+#ifndef mksh_siglist
+#if defined(BSD) || defined(__APPLE__)
+#define	mksh_signame(x)	sys_signame[(x)]
+#define	mksh_siglist(x)	sys_siglist[(x)]
+#elif defined(__INTERIX)
+#define	mksh_signame(x)	_sys_signame[(x)]
+#define	mksh_siglist(x) _sys_siglist[(x)]
+#elif defined(__gnu_linux__) || defined(__sun__) || defined(__CYGWIN__)
+#define	NEED_MKSH_SIGNAME	/* sync the list above with Build.sh */
+#define	mksh_siglist(x)	strsignal(x)
+#else
+# error "Define sys_sig{name,list} for this platform!"
+#endif
+#endif /* ndef mksh_siglist */
+
+Trap sigtraps[NSIG + 1];
+static struct sigaction Sigact_ign, Sigact_trap;
+
+#ifdef DO_HISTORY
 static int hist_count_lines(unsigned char *, int);
 static int hist_shrink(unsigned char *, int);
 static unsigned char *hist_skip_back(unsigned char *,int *,int);
@@ -29,22 +48,8 @@ static char    *hname;		/* current name of history file */
 static int	hstarted;	/* set after hist_init() called */
 static Source	*hist_source;
 
-#if defined(BSD) || defined(__APPLE__)
-#define	mksh_signame(x)	sys_signame[(x)]
-#define	mksh_siglist(x)	sys_siglist[(x)]
-#elif defined(__INTERIX)
-#define	mksh_signame(x)	_sys_signame[(x)]
-#define	mksh_siglist(x) _sys_siglist[(x)]
-#elif defined(__gnu_linux__) || defined(__sun__) || defined(__CYGWIN__)
-#define	NEED_MKSH_SIGNAME
-#define	mksh_siglist(x)	strerror(x)
-#else
-# error "Define sys_sig{name,list} for this platform!"
-#endif
-
-Trap sigtraps[NSIG + 1];
-
-static struct sigaction Sigact_ign, Sigact_trap;
+static int	histfd;
+static int	hsize;
 
 int
 c_fc(char **wp)
@@ -583,7 +588,7 @@ histsave(int lno __attribute__((unused)), const char *cmd,
 	if ((cp = strchr(c, '\n')) != NULL)
 		*cp = '\0';
 
-#ifndef __sun__
+#ifdef DO_HISTORY
 	if (histfd && dowrite)
 		writehistfile(lno, c);
 #endif
@@ -626,7 +631,7 @@ histsave(int lno __attribute__((unused)), const char *cmd,
 void
 hist_init(Source *s)
 {
-#ifndef __sun__
+#ifdef DO_HISTORY
 	unsigned char	*base;
 	int	lines;
 	int	fd;
@@ -639,16 +644,13 @@ hist_init(Source *s)
 
 	hist_source = s;
 
-#ifdef __sun__
-	hname = NULL;
-	histfd = 0;
-#else
+#ifdef DO_HISTORY
 	hname = str_val(global("HISTFILE"));
 	if (hname == NULL)
 		return;
 	hname = str_save(hname, APERM);
 
-  retry:
+ retry:
 	/* we have a file and are interactive */
 	if ((fd = open(hname, O_RDWR|O_CREAT|O_APPEND, 0600)) < 0)
 		return;
@@ -667,8 +669,7 @@ hist_init(Source *s)
 			hist_finish();
 			return;
 		}
-	}
-	else if (hsize > 0) {
+	} else if (hsize > 0) {
 		/*
 		 * we have some data
 		 */
@@ -700,6 +701,9 @@ hist_init(Source *s)
 	}
 	(void) flock(histfd, LOCK_UN);
 	hsize = lseek(histfd, 0L, SEEK_END);
+#else
+	hname = NULL;
+	histfd = 0;
 #endif
 }
 
@@ -710,7 +714,7 @@ typedef enum state {
 	sn2, sn3, sn4
 } State;
 
-#ifndef __sun__
+#ifdef DO_HISTORY
 static int
 hist_count_lines(unsigned char *base, int bytes)
 {
@@ -856,8 +860,7 @@ histload(Source *s, unsigned char *base, int bytes)
 				if (histptr >= history && lno-1 != s->line) {
 					/* a replacement ? */
 					histinsert(s, lno, line);
-				}
-				else {
+				} else {
 					s->line = lno;
 					histsave(lno, (char *)line, 0);
 				}
@@ -943,7 +946,7 @@ writehistfile(int lno, char *cmd)
 	hsize = lseek(histfd, 0L, SEEK_END);
 	(void) flock(histfd, LOCK_UN);
 	return;
-bad:
+ bad:
 	hist_finish();
 }
 #endif
@@ -951,14 +954,14 @@ bad:
 void
 hist_finish(void)
 {
-#ifndef __sun__
+#ifdef DO_HISTORY
 	(void) flock(histfd, LOCK_UN);
 	(void) close(histfd);
 #endif
 	histfd = 0;
 }
 
-#ifndef __sun__
+#ifdef DO_HISTORY
 /*
  *	add magic to the history file
  */
@@ -984,12 +987,11 @@ mksh_signame(int s)
 		{ 0, NULL }
 	};
 
-	while (mksh_sigpair[i].name != NULL) {
-		if (mksh_sigpair[i].nr == s)
-			return mksh_sigpair[i].name;
-		++i;
-	}
-	return NULL;
+ mksh_sigscan:
+	if ((mksh_sigpair[i].nr == s) || !mksh_sigpair[i].name)
+		return (mksh_sigpair[i].name);
+	++i;
+	goto mksh_sigscan;
 }
 #endif
 
