@@ -1,8 +1,8 @@
-/*	$OpenBSD: lex.c,v 1.39 2006/04/10 14:38:59 jaredy Exp $	*/
+/*	$OpenBSD: lex.c,v 1.42 2006/07/10 17:12:41 beck Exp $	*/
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/lex.c,v 1.13 2006/07/03 12:16:30 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/lex.c,v 1.21 2006/08/02 11:33:37 tg Exp $");
 
 /* Structure to keep track of the lexing state and the various pieces of info
  * needed for each particular state. */
@@ -58,6 +58,8 @@ static void	gethere(void);
 static Lex_state *push_state_(State_info *, Lex_state *);
 static Lex_state *pop_state_(State_info *, Lex_state *);
 
+static int dopprompt(const char *, int, int);
+
 static int backslash_skip;
 static int ignore_backslash_newline;
 
@@ -102,8 +104,7 @@ yylex(int cf)
 	char *sp, *dp;
 	int c2;
 
-
-  Again:
+ Again:
 	states[0].ls_state = -1;
 	states[0].ls_info.base = NULL;
 	statep = &states[1];
@@ -178,7 +179,7 @@ yylex(int cf)
 				break;
 			}
 			/* FALLTHRU */
-		  Sbase1:	/* includes *(...|...) pattern (*+?@!) */
+ Sbase1:		/* includes *(...|...) pattern (*+?@!) */
 			if (c == '*' || c == '@' || c == '+' || c == '?' ||
 			    c == '!') {
 				c2 = getsc();
@@ -191,7 +192,7 @@ yylex(int cf)
 				ungetsc(c2);
 			}
 			/* FALLTHRU */
-		  Sbase2:	/* doesn't include *(...|...) pattern (*+?@!) */
+ Sbase2:		/* doesn't include *(...|...) pattern (*+?@!) */
 			switch (c) {
 			case '\\':
 				c = getsc();
@@ -212,7 +213,7 @@ yylex(int cf)
 			}
 			break;
 
-		  Subst:
+ Subst:
 			switch (c) {
 			case '\\':
 				c = getsc();
@@ -226,7 +227,7 @@ yylex(int cf)
 					*wp++ = QCHAR, *wp++ = c;
 					break;
 				default:
-heredocquote:
+ heredocquote:
 					Xcheck(ws, wp);
 					if (c) { /* trailing \ is lost */
 						*wp++ = CHAR, *wp++ = '\\';
@@ -601,7 +602,7 @@ heredocquote:
 			break;
 		}
 	}
-Done:
+ Done:
 	Xcheck(ws, wp);
 	if (statep != &states[1])
 		/* XXX figure out what is missing */
@@ -1086,12 +1087,58 @@ set_prompt(int to, Source *s)
 	}
 }
 
-/* See also related routine, promptlen() in edit.c */
+static int
+dopprompt(const char *cp, int ntruncate, int doprint)
+{
+	int columns = 0, lines = 0, indelimit = 0;
+	char delimiter = 0;
+
+	/* Undocumented AT&T ksh feature:
+	 * If the second char in the prompt string is \r then the first char
+	 * is taken to be a non-printing delimiter and any chars between two
+	 * instances of the delimiter are not considered to be part of the
+	 * prompt length
+	 */
+	if (*cp && cp[1] == '\r') {
+		delimiter = *cp;
+		cp += 2;
+	}
+	for (; *cp; cp++) {
+		if (indelimit && *cp != delimiter)
+			;
+		else if (*cp == '\n' || *cp == '\r') {
+			lines += columns / x_cols + ((*cp == '\n') ? 1 : 0);
+			columns = 0;
+		} else if (*cp == '\t') {
+			columns = (columns | 7) + 1;
+		} else if (*cp == '\b') {
+			if (columns > 0)
+				columns--;
+		} else if (*cp == delimiter)
+			indelimit = !indelimit;
+		else
+			columns++;
+		if (doprint && (*cp != delimiter) &&
+		    (indelimit || (ntruncate < (x_cols * lines + columns))))
+			shf_putc(*cp, shl_out);
+	}
+	if (doprint)
+		shf_flush(shl_out);
+	indelimit = (x_cols * lines + columns);
+	return indelimit;
+}
+
+
 void
 pprompt(const char *cp, int ntruncate)
 {
-	shf_puts(cp + ntruncate, shl_out);
-	shf_flush(shl_out);
+	dopprompt(cp, ntruncate, 1);
+}
+
+int
+promptlen(const char *cp)
+{
+	return (dopprompt(cp, 0, 0));
 }
 
 /* Read the variable part of a ${...} expression (ie, up to but not including
