@@ -3,24 +3,22 @@
 /*	$OpenBSD: io.c,v 1.22 2006/03/17 16:30:13 millert Exp $	*/
 /*	$OpenBSD: table.c,v 1.12 2005/12/11 20:31:21 otto Exp $	*/
 
-#define	EXTERN				/* define EXTERNs in sh.h */
+#define	EXTERN
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/main.c,v 1.40 2006/07/03 12:16:30 tg Exp $");
-
-#define	MKSH_VERSION	"@(#)MIRBSD KSH R27 2006/07/03"
+__RCSID("$MirOS: src/bin/mksh/main.c,v 1.48 2006/08/24 20:32:53 tg Exp $");
 
 extern char **environ;
 
 static void reclaim(void);
-static void remove_temps(struct temp * tp);
+static void remove_temps(struct temp *);
 
 static const char initifs[] = "IFS= \t\n";
 
 static const char initsubs[] = "${PS2=> } ${PS3=#? } ${PS4=+ }";
 
 static const char *initcoms[] = {
-	"typeset", "-r", "KSH_VERSION", NULL,
+	"typeset", "-r", initvsn, NULL,
 	"typeset", "-x", "SHELL", "PATH", "HOME", NULL,
 	"typeset", "-i", "PPID", "OPTIND=1", NULL,
 	"eval", "typeset -i RANDOM SECONDS=\"${SECONDS-0}\" TMOUT=\"${TMOUT-0}\"", NULL,
@@ -116,7 +114,7 @@ main(int argc, char *argv[])
 	init_histvec();
 
 	def_path = _PATH_DEFPATH;
-#if !defined(__CYGWIN__)
+#ifdef _CS_PATH
 	{
 		size_t len;
 		char *new;
@@ -193,8 +191,6 @@ main(int argc, char *argv[])
 	srand((*((long *)kshname)) ^ ((long)time(NULL) * kshpid * ppid));
 #endif
 	setint(global("PPID"), (long)ppid);
-	/* setstr can't fail here */
-	setstr(global("KSH_VERSION"), MKSH_VERSION, KSH_RETURN_ERROR);
 
 	/* execute initialisation statements */
 	for (wp = (char **)initcoms; *wp != NULL; wp++) {
@@ -1163,7 +1159,7 @@ ktenter(struct table *tp, const char *n, unsigned int h)
 
 	if (tp->size == 0)
 		texpand(tp, INIT_TBLS);
-Search:
+ Search:
 	/* search for name in hashed table */
 	for (pp = &tp->tbls[h & (tp->size - 1)]; (p = *pp) != NULL; pp--) {
 		if (*p->name == *n && strcmp(p->name, n) == 0)
