@@ -3,8 +3,8 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/misc.c,v 1.11 2006/05/10 18:54:11 tg Exp $"
-	"\t" MKSH_SH_H_ID);
+__RCSID("$MirOS: src/bin/mksh/misc.c,v 1.16 2006/08/24 20:32:53 tg Exp $\t"
+	MKSH_SH_H_ID);
 
 short chtypes[UCHAR_MAX+1];	/* type bits for unsigned char */
 
@@ -12,8 +12,7 @@ static int do_gmatch(const unsigned char *, const unsigned char *,
     const unsigned char *, const unsigned char *);
 static const unsigned char *cclass(const unsigned char *, int);
 static int parse_T(char *);
-
-static char	*do_phys_path(XString *, char *, const char *);
+static char *do_phys_path(XString *, char *, const char *);
 
 /*
  * Fast character classes
@@ -248,8 +247,8 @@ change_flag(enum sh_flag f,
 {
 	int oldval;
 
-	/* disabled functionality */
-	if ((f == FPOSIX) || (f == FSH))
+	/* limited pdksh compatibility (FSH stays always off) */
+	if (f == FSH)
 		return;
 
 	oldval = Flag(f);
@@ -257,18 +256,17 @@ change_flag(enum sh_flag f,
 	if (f == FMONITOR) {
 		if (what != OF_CMDLINE && newval != oldval)
 			j_change();
-	} else if (f == FVI || f == FEMACS || f == FGMACS) {
-		if (newval) {
-			Flag(FVI) = 0;
-			Flag(FEMACS) = Flag(FGMACS) = 0;
-			Flag(f) = newval;
-		}
+	} else if ((f == FVI || f == FEMACS || f == FGMACS) && newval) {
+		Flag(FVI) = Flag(FEMACS) = Flag(FGMACS) = 0;
+		Flag(f) = newval;
 	} else if (f == FPRIVILEGED && oldval && !newval) {
 		/* Turning off -p? */
 		seteuid(ksheuid = kshuid = getuid());
 		setuid(ksheuid);
 		setegid(kshegid = kshgid = getgid());
 		setgid(kshegid);
+	} else if (f == FPOSIX && newval) {
+		Flag(FBRACEEXPAND) = 0;
 	}
 	/* Changing interactive flag? */
 	if (f == FTALKING) {
@@ -778,7 +776,7 @@ qsort1(void **base, void **lim, int (*f) (void *, void *))
 	size_t n;
 	int c;
 
-  top:
+ top:
 	n = (lim - base) / 2;
 	if (n == 0)
 		return;
@@ -799,7 +797,7 @@ qsort1(void **base, void **lim, int (*f) (void *, void *))
 			}
 		}
 
-	  begin:
+ begin:
 		if (j > hptr) {
 			if ((c = (*f)(*hptr, *j)) == 0) {
 				hptr ++;
@@ -1131,7 +1129,7 @@ reset_nonblock(int fd)
 }
 
 
-/* Like getcwd(), except bsize is ignored if buf is 0 (MAXPATHLEN is used) */
+/* Like getcwd(), except bsize is ignored if buf is 0 (PATH_MAX is used) */
 char *
 ksh_get_wd(char *buf, int bsize)
 {
@@ -1142,8 +1140,8 @@ ksh_get_wd(char *buf, int bsize)
 	 * inject possibly allocated space into the ATEMP area. */
 	/* Assume getcwd() available */
 	if (!buf) {
-		bsize = MAXPATHLEN;
-		b = alloc(MAXPATHLEN + 1, ATEMP);
+		bsize = PATH_MAX;
+		b = alloc(PATH_MAX + 1, ATEMP);
 	} else
 		b = buf;
 
