@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/var.c,v 1.20 2006/07/03 12:16:31 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/var.c,v 1.23 2006/08/18 18:48:26 tg Exp $");
 
 /*
  * Variables
@@ -530,11 +530,11 @@ formatstr(struct tbl *vp, const char *s)
 	if (vp->flag & UCASEV_AL) {
 		for (q = p; *q; q++)
 			if (islower((unsigned char)*q))
-				*q = toupper(*q);
+				*q = toupper((unsigned char)*q);
 	} else if (vp->flag & LCASEV) {
 		for (q = p; *q; q++)
 			if (isupper((unsigned char)*q))
-				*q = tolower(*q);
+				*q = tolower((unsigned char)*q);
 	}
 
 	return p;
@@ -917,10 +917,10 @@ getspec(struct tbl *vp)
 #if HAVE_ARC4RANDOM
 		if (use_rand)
 #endif
-			setint(vp, (long) (rand() & 0x7fff));
+			setint(vp, (long) (rand() & 0x7FFF));
 #if HAVE_ARC4RANDOM
 		else
-			setint(vp, (long) (arc4random() & 0x7fff));
+			setint(vp, arc4random() & 0x7FFFFFFF);
 #endif
 		vp->flag |= SPECIAL;
 		break;
